@@ -1,7 +1,8 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * gsf-input-iochannel.h: interface for use by the structured file layer to read data from IO channels
  *
- * Copyright (C) 2002-2003 Rodrigo Moya (rodrigo@gnome-db.org)
+ * Copyright (C) 2002-2004 Rodrigo Moya (rodrigo@gnome-db.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -14,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -25,8 +26,8 @@
 
 G_BEGIN_DECLS
 
-GsfInputMemory * gsf_input_memory_new_from_iochannel (GIOChannel * channel,
-						      GError ** error);
+GsfInput *gsf_input_memory_new_from_iochannel (GIOChannel *channel,
+					       GError    **error);
 
 G_END_DECLS
 
