@@ -139,7 +139,7 @@ od_meta_keyword (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 }
 
 static void
-od_meta_user_defined (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
+od_meta_user_defined (G_GNUC_UNUSED GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
 }
 
@@ -316,15 +316,28 @@ meta_write_props (char const *prop_name, GsfDocProp *prop, GsfXMLOut *output)
 
 	/* Handle specially */
 	if (0 == strcmp (prop_name, GSF_META_NAME_KEYWORDS)) {
-		GValueArray *va = gsf_value_get_docprop_varray (val);
+		GValueArray *va;
 		unsigned i;
 		char *str;
-		
-		for (i = 0 ; i < va->n_values; i++) {
-			str = g_value_dup_string (g_value_array_get_nth	(va, i));
-			gsf_xml_out_start_element (output, "meta:keyword");
-			gsf_xml_out_add_cstr (output, NULL, str);
+
+		/* OLE2 stores a single string, with no obvious
+		 * standard for seperator */
+		if (G_TYPE_STRING == G_VALUE_TYPE (val)) {
+			str = g_value_dup_string (val);
+			if (str && *str) {
+				gsf_xml_out_start_element (output, "meta:keyword");
+				gsf_xml_out_add_cstr (output, NULL, str);
+				gsf_xml_out_end_element (output);
+			}
 			g_free (str);
+		} else if (NULL != (va = gsf_value_get_docprop_varray (val))) {
+			for (i = 0 ; i < va->n_values; i++) {
+				str = g_value_dup_string (g_value_array_get_nth	(va, i));
+				gsf_xml_out_start_element (output, "meta:keyword");
+				gsf_xml_out_add_cstr (output, NULL, str);
+				gsf_xml_out_end_element (output);
+				g_free (str);
+			}
 		}
 		return;
 	}
@@ -336,8 +349,10 @@ meta_write_props (char const *prop_name, GsfDocProp *prop, GsfXMLOut *output)
 		gsf_xml_out_start_element (output, "meta:user-defined");
 		gsf_xml_out_add_cstr (output, "meta:name", prop_name);
 
-		if (NULL == val)
+		if (NULL == val) {
 			gsf_xml_out_end_element (output);
+			return;
+		}
 
 		switch ((t = G_VALUE_TYPE (val))) {
 		case G_TYPE_CHAR:
@@ -363,10 +378,12 @@ meta_write_props (char const *prop_name, GsfDocProp *prop, GsfXMLOut *output)
 			if (GSF_TIMESTAMP_TYPE == t)
 				type_name = "data";
 		}
-		gsf_xml_out_add_cstr (output, "meta:type", type_name);
+		if (NULL != type_name)
+			gsf_xml_out_add_cstr (output, "meta:type", type_name);
 	} else
 		gsf_xml_out_start_element (output, mapped_name);
-	gsf_xml_out_add_gvalue (output, NULL, val);
+	if (NULL != val)
+		gsf_xml_out_add_gvalue (output, NULL, val);
 	gsf_xml_out_end_element (output);
 }
 
