@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -256,9 +256,6 @@ gsf_output_bzip_new (GsfOutput *sink, GError **err)
 
 	return GSF_OUTPUT (bzip);
 #else
-#ifdef __GNUC__
-#warning Building without BZ2 support
-#endif
 	if (err)
 		*err = g_error_new (gsf_output_error_id (), 0,
 				    "BZ2 support not enabled");
