@@ -24,6 +24,9 @@
 #include <string.h>
 #include <time.h>
 #include <stdio.h>
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
 
 static void
 timestamp_to_string (GValue const *src_value, GValue *dest_value)
@@ -80,15 +83,15 @@ gsf_timestamp_free (GsfTimestamp *stamp)
 #define GMTOFF(t) ((t).tm_gmtoff)
 #elif defined(HAVE_STRUCT_TM___TM_GMTOFF)
 #define GMTOFF(t) ((t).__tm_gmtoff)
-#elif defined(WIN32)
+#elif defined(G_OS_WIN32)
 #define GMTOFF(t) (gmt_to_local_win32())
 #else
 /* FIXME: work out the offset anyway. */
 #define GMTOFF(t) (0)
 #endif
 
-#ifdef WIN32
-time_t gmt_to_local_win32(void)
+#ifdef G_OS_WIN32
+static time_t gmt_to_local_win32(void)
 {
     TIME_ZONE_INFORMATION tzinfo;
     DWORD dwStandardDaylight;
@@ -108,15 +111,17 @@ time_t gmt_to_local_win32(void)
 #endif
 
 /**
- * gsf_timestamp_parse : ICK ICK ICK
- * Should be called _from_stong.
+ * gsf_timestamp_parse :
  * @spec : The string to parse
  * @stamp : #GsfTimestamp
  *
- * Very simple parser for time stamps.  Currently requires
+ * Very simple parser for time stamps.  Currently requires a format of
  * 	'YYYY-MM-DDThh:mm:ss'
  * and does no bounds checking.
  *
+ * ICK ICK ICK
+ * This routine should be called _from_sting.
+ *
  * Returns TRUE on success
  **/
 int
@@ -146,7 +151,7 @@ gsf_timestamp_parse (char const *spec, GsfTimestamp *stamp)
  * gsf_timestamp_as_string :
  * @stamp: timestamp to be converted.
  *
- * Produce a string representation of @stamp.
+ * Produce a string representation (ISO 8601 format) of @stamp.
  *
  * Returns: a string representation of @stamp. When @stamp is NULL, the
  * representation is "&lt;invalid&gt;".
@@ -160,10 +165,16 @@ gsf_timestamp_as_string	(GsfTimestamp const *stamp)
 	g_return_val_if_fail (stamp != NULL, g_strdup ("<invalid>"));
 
 	t = stamp->timet;	/* Use an honest time_t for gmtime_r.  */
+#ifdef HAVE_GMTIME_R
 	gmtime_r (&t, &tm);
+#else
+	/* -NOT- thread-safe */
+	tm = *gmtime (&t);
+#endif
+
 
 	/* using 'YYYY-MM-DDThh:mm:ss' */
-	return g_strdup_printf ("%4d-%02d-%02dT%02d:%02d:%d",
+	return g_strdup_printf ("%4d-%02d-%02dT%02d:%02d:%02dZ",
 		tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday,
 		tm.tm_hour, tm.tm_min, tm.tm_sec);
 }
