@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -41,10 +41,6 @@ struct _GsfTimestamp {
 
 GType gsf_timestamp_get_type (void);
 
-GsfTimestamp *gsf_timestamp_new_now	(void);
-GsfTimestamp *gsf_timestamp_new		(GDate const *date, glong seconds,
-					 char const *tz);
-
 GsfTimestamp *gsf_timestamp_copy	(GsfTimestamp const *stamp);
 void          gsf_timestamp_free	(GsfTimestamp       *stamp);
 int           gsf_timestamp_parse	(char const *spec, GsfTimestamp *stamp);
